﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32.SafeHandles;
using System.Runtime.InteropServices;
using System.IO;

namespace USBHelper
{
    public class USBRawReader
    {
        const uint GENERIC_READ = 0x80000000;
        const uint OPEN_EXISTING = 3;
        const int FILE_SHARE_READ = 0x00000001;
        const int FILE_SHARE_WRITE = 0x00000002;

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern SafeFileHandle CreateFile(
            string lpFileName,
            uint dwDesiredAccess,
            int dwShareMode,
            IntPtr lpSecurityAttributes,
            uint dwCreationDisposition,
            int dwFlagsAndAttributes,
            IntPtr hTemplateFile);

        // public static byte[] ReadSector(string physicalDrivePath, int sectorNumber = 2)
        public static string ReadSector(string physicalDrivePath, int sectorNumber = 2)
        {
            const int SECTOR_SIZE = 512;
            byte[] buffer = new byte[SECTOR_SIZE];

            SafeFileHandle handle = CreateFile(
                physicalDrivePath,
                GENERIC_READ,
                FILE_SHARE_READ | FILE_SHARE_WRITE,
                IntPtr.Zero,
                OPEN_EXISTING,
                0,
                IntPtr.Zero);

            if (handle.IsInvalid)
            {
                throw new IOException("Failed to open drive. Run as Administrator.");
            }

            using (FileStream stream = new FileStream(handle, FileAccess.Read))
            {
                stream.Seek(SECTOR_SIZE * sectorNumber, SeekOrigin.Begin);
                int bytesRead = stream.Read(buffer, 0, SECTOR_SIZE);
                if (bytesRead != SECTOR_SIZE)
                {
                    throw new IOException("Failed to read full sector.");
                }
            }

            string hexString = BitConverter.ToString(buffer);  // "1D-00-..."
            return hexString.Replace("-", " ");                 // "1D 00 ..."
        }
    }
}
