﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;
using System.IO;

namespace USBHelper
{
    public class USBRawWriter
    {
        const uint GENERIC_WRITE = 0x40000000;
        const uint OPEN_EXISTING = 3;
        const int FILE_SHARE_READ = 0x00000001;
        const int FILE_SHARE_WRITE = 0x00000002;

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern SafeFileHandle CreateFile(
            string lpFileName,
            uint dwDesiredAccess,
            int dwShareMode,
            IntPtr lpSecurityAttributes,
            uint dwCreationDisposition,
            int dwFlagsAndAttributes,
            IntPtr hTemplateFile);

        // Add this to USBRawWriter class:
        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool FlushFileBuffers(IntPtr hFile);

        public static bool WriteSector(string physicalDrivePath, int sectorNumber, byte[] data)
        {
            const int SECTOR_SIZE = 512;

            if (data == null || data.Length != SECTOR_SIZE)
                throw new ArgumentException("Data must be exactly " + SECTOR_SIZE + " bytes");

            using (SafeFileHandle handle = CreateFile(
                physicalDrivePath,
                GENERIC_WRITE,
                FILE_SHARE_READ | FILE_SHARE_WRITE,
                IntPtr.Zero,
                OPEN_EXISTING,
                0,
                IntPtr.Zero))
            {
                if (handle.IsInvalid)
                    return false;

                using (FileStream stream = new FileStream(handle, FileAccess.Write))
                {
                    stream.Seek(SECTOR_SIZE * sectorNumber, SeekOrigin.Begin);
                    stream.Write(data, 0, SECTOR_SIZE);
                    stream.Flush();

                    if (!FlushFileBuffers(handle.DangerousGetHandle()))
                    {
                        int err = Marshal.GetLastWin32Error();
                        Console.WriteLine("FlushFileBuffers failed: error " + err, "Error");
                        return false;
                    }
                }
            }
            return true;
        }
    }
}
