﻿using System;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.Win32.SafeHandles;
using System.Management;
using System.Drawing;
using System.Diagnostics;
using System.Security.Principal;
using System.Drawing.Drawing2D;
using System.Text;
using USBWPRTC;
using USBHelper;

namespace USBWriteProtect
{
    public partial class Form1 : Form
    {
        #region Variables

        private ComboBox driveComboBox;

        private Button lockButton;
        private Button unLockButton;
        private Button readButton;

        private TextBox pswTextBox;

        private Label headerLabel;
        private Label footerLabel;
        private Label adminLabel;

        private PictureBox topRightIcon;
        private string iconPath = Path.Combine(Application.StartupPath, "resources", "shieldok.ico");

        private Panel paddedPswActionsPanel;
        private Panel paddedActionsPanel;
        private Panel paddedDrivesPanel;

        private bool blnLock = true;

        private const int DEFAULT_SECTOR_NUMBER = 2;

        private ManagementEventWatcher insertWatcher;
        private ManagementEventWatcher removeWatcher;

        private string defaultAdminPass = "Admin1234";

        #endregion

        #region Constructure Settings

        public Form1()
        {

            #region InitializeComponent

            InitializeComponent();

            // USB Watchers
            StartUSBWatchers();

            #endregion

            #region Form Settings

            // 🔷 Form appearance and behavior
            this.StartPosition = FormStartPosition.CenterScreen;
            this.MaximizeBox = false;
            this.ShowIcon = true;
            this.ShowInTaskbar = true;
            this.WindowState = FormWindowState.Normal;
            this.AutoScaleMode = AutoScaleMode.None; // fixed layout scaling
            this.Size = new System.Drawing.Size(300, 255);
            this.Font = new Font("Tahoma", 10);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;

            // 🔷 Keyboard event support
            this.KeyPreview = true; // lets form capture key presses before controls
            this.KeyDown += Form1_KeyDown;
            this.Load += Form1_Load;
            this.Shown += Form1_Shown;
            this.FormClosing += OnFormClosing;

            // 🔷 App Text
            this.Text = "USB Write Protect";

            #endregion

            #region Controls
            // Header
            headerLabel = new Label();
            headerLabel.Text = this.Text;
            headerLabel.AutoSize = false;
            headerLabel.Height = 35;
            headerLabel.BackColor = Color.SteelBlue;
            headerLabel.ForeColor = Color.WhiteSmoke;
            headerLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
            headerLabel.Dock = DockStyle.Top;
            headerLabel.Font = new Font("Tahoma", 10, FontStyle.Bold);

            #region ADMIN Logo

            // Create the PictureBox
            topRightIcon = new PictureBox();
            topRightIcon.Size = new Size(24, 24);
            topRightIcon.SizeMode = PictureBoxSizeMode.StretchImage;
            // FIXED: Transparent background (use Color.Transparent, not 'Transparent' directly)
            topRightIcon.BackColor = Color.SteelBlue;

            adminLabel = new Label();
            adminLabel.Text = "ADMIN";
            adminLabel.AutoSize = true;
            adminLabel.Font = new Font("Tahoma", 8, FontStyle.Bold);
            adminLabel.ForeColor = Color.Gold;
            adminLabel.BackColor = Color.SteelBlue;
            // Position it in top-right corner
            adminLabel.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            adminLabel.Location = new Point(this.ClientSize.Width - 45, 1);
            adminLabel.Visible = false;

            if (File.Exists(iconPath))
            {
                // Use a memory stream to avoid disposing the icon prematurely
                using (Icon icon = new Icon(iconPath))
                {
                    topRightIcon.Image = icon.ToBitmap();  // Convert .ico to .bmp for PictureBox
                }

                // Position at top-right corner
                topRightIcon.Location = new Point(this.ClientSize.Width - topRightIcon.Width - 5, 5);
                topRightIcon.Anchor = AnchorStyles.Top | AnchorStyles.Right;

                // Optional: click behavior
                topRightIcon.Cursor = Cursors.Default;
                // topRightIcon.Click += (s, e) => MessageBox.Show("Shield icon clicked!");

                topRightIcon.Visible = false;

                // Add to the form
                this.Controls.Add(topRightIcon);
            }
            else
            {
                // icon not found.
                this.Controls.Add(adminLabel);
            }

            #endregion

            // Footer
            footerLabel = new Label();
            footerLabel.Text = "Copyright © Mammoth™ 2025";
            footerLabel.AutoSize = false;
            footerLabel.Height = 20;
            footerLabel.BackColor = Color.LightGray;
            footerLabel.ForeColor = Color.SteelBlue;
            footerLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
            footerLabel.Dock = DockStyle.Bottom;
            footerLabel.Font = new Font("Tahoma", 7, FontStyle.Bold);

            paddedDrivesPanel = new Panel();
            paddedDrivesPanel.Dock = DockStyle.Top;
            // Set font size first (optional, improves appearance with taller height)
            paddedDrivesPanel.Font = new Font("Tahoma", 10);
            paddedDrivesPanel.Padding = new Padding(11, 10, 11, 0); // padding inside the panel
            paddedDrivesPanel.Height = 35; // adjust as needed
            paddedDrivesPanel.BorderStyle = BorderStyle.None;

            // Create ComboBox
            driveComboBox = new ComboBox();
            driveComboBox.ItemHeight = 35;
            driveComboBox.Dock = DockStyle.Top;
            driveComboBox.DropDownStyle = ComboBoxStyle.DropDownList; // DropDown or DropDownList

            // Set font size first (optional, improves appearance with taller height)
            driveComboBox.Font = new Font("Tahoma", 10);

            // Then set height directly
            driveComboBox.Height = 18;

            // Optional: affect dropdown item spacing too
            driveComboBox.ItemHeight = 18;

            // Populate drive letters
            try
            {
                foreach (var drive in DriveInfo.GetDrives())
                {
                    if (drive.IsReady && drive.DriveType == DriveType.Removable)
                    {
                        driveComboBox.Items.Add(drive.Name.TrimEnd('\\') + " " + drive.VolumeLabel.ToString());
                    }
                }

                if (driveComboBox.Items.Count > 0) driveComboBox.SelectedIndex = 0; // select first drive by default
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message, "Error !", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                // // Prevent any control from being focused
                this.setNonActiveControl();
            }
            finally
            {
                if (driveComboBox.Items.Count > 0)
                {
                    driveComboBox.Enabled = true;
                    driveComboBox.SelectedIndex = 0;
                }
                else
                {
                    driveComboBox.Enabled = false;
                }
            }

            paddedPswActionsPanel = new Panel();
            paddedPswActionsPanel.Dock = DockStyle.Top;
            // Set font size first (optional, improves appearance with taller height)
            paddedPswActionsPanel.Font = new Font("Tahoma", 10);
            paddedPswActionsPanel.Padding = new Padding(11, 5, 11, 0); // padding inside the panel
            paddedPswActionsPanel.Height = 35; // adjust as needed
            paddedPswActionsPanel.BorderStyle = BorderStyle.None;

            paddedActionsPanel = new Panel();
            paddedActionsPanel.Dock = DockStyle.Top;
            // Set font size first (optional, improves appearance with taller height)
            paddedActionsPanel.Font = new Font("Tahoma", 10);
            paddedActionsPanel.Padding = new Padding(10, 5, 10, 0); // padding inside the panel
            paddedActionsPanel.Height = 90; // adjust as needed
            paddedActionsPanel.BorderStyle = BorderStyle.None;

            pswTextBox = new TextBox();
            pswTextBox.Height = 25;
            pswTextBox.Dock = DockStyle.Top;
            pswTextBox.Font = new Font("Tahoma", 10);
            pswTextBox.TextAlign = HorizontalAlignment.Center;
            pswTextBox.MaxLength = 64;
            //pswTextBox.PasswordChar = '•';
            pswTextBox.PasswordChar = '\0';           // Disable masking (show text normally)
            pswTextBox.Text = "Welcome to " + this.Text;
            pswTextBox.Enabled = false;               // Optionally disable to prevent further editing
            // 🔷 Add KeyDown event handler
            pswTextBox.KeyDown += pswTextBox_KeyDown;

            // Add buttons after comboBox (add in reverse order to get proper stacking)
            readButton = new Button();
            readButton.Height = 25;
            readButton.Text = "Read data";
            readButton.Dock = DockStyle.Top;
            //readButton.Click += ReadSector;
            readButton.Click += (s, evt) => ReadSector(s, evt, DEFAULT_SECTOR_NUMBER); // assignment using lambda

            lockButton = new Button();
            lockButton.Height = 25;
            lockButton.Text = "Lock USB";
            lockButton.Dock = DockStyle.Top;
            lockButton.Click += (s, evt) => LockUSB(s, evt, DEFAULT_SECTOR_NUMBER); // assignment using lambda
            lockButton.Enabled = false; // disabled by default

            unLockButton = new Button();
            unLockButton.Height = 25;
            unLockButton.Text = "UnLock USB";
            unLockButton.Dock = DockStyle.Top;
            unLockButton.Click += (s, evt) => UnLockUSB(s, evt, DEFAULT_SECTOR_NUMBER); // assignment using lambda;
            unLockButton.Enabled = false; // disabled by default

            // Footer Label
            Controls.Add(footerLabel);

            // Add Action buttons to panel
            paddedActionsPanel.Controls.Add(readButton);
            paddedActionsPanel.Controls.Add(unLockButton);
            paddedActionsPanel.Controls.Add(lockButton);
            // Add paddedActionsPanel to form
            Controls.Add(paddedActionsPanel);

            // Add Psw Action TextBox to panel
            paddedPswActionsPanel.Controls.Add(pswTextBox);
            // Add paddedPswActionsPanel to form
            Controls.Add(paddedPswActionsPanel);

            // Add ComboBox to controls
            paddedDrivesPanel.Controls.Add(driveComboBox);
            // Add paddedDrivesPanel to form
            Controls.Add(paddedDrivesPanel);

            // Add Header Label
            Controls.Add(headerLabel);

            #endregion

        }

        #endregion

        #region Form and Controls Events

        private void setNonActiveControl()
        {
            // // Prevent any control from being focused
            this.BeginInvoke(new Action(() => this.ActiveControl = null));
        }

        private bool IsAdministrator()
        {
            WindowsIdentity identity = WindowsIdentity.GetCurrent();
            WindowsPrincipal principal = new WindowsPrincipal(identity);
            return principal.IsInRole(WindowsBuiltInRole.Administrator);
        }

        protected override void OnActivated(EventArgs e)
        {
            base.OnActivated(e);
            this.BeginInvoke(new Action(() => this.ActiveControl = null));
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Check for admin privileges
            if (!IsAdministrator())
            {
                try
                {
                    // Restart application with elevated privileges
                    ProcessStartInfo proc = new ProcessStartInfo
                    {
                        UseShellExecute = true,
                        WorkingDirectory = Environment.CurrentDirectory,
                        FileName = Application.ExecutablePath,
                        Verb = "runas" // This triggers UAC prompt
                    };

                    Process.Start(proc);
                }
                catch
                {
                    MessageBox.Show("Administrator privileges are required to run this application.",
                        "Permission Required", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                // Exit current non-admin instance
                Application.Exit();
                return;
            }

            // if privileges is admin, show admin logo / text
            if (File.Exists(iconPath))
            {
                topRightIcon.Visible = true;
            }
            else
            {
                adminLabel.Visible = true;
            }

            // Enable or disable the read button based on driveComboBox item count
            readButton.Enabled = (driveComboBox != null && driveComboBox.Items.Count > 0);
        }

        private void Form1_Shown(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
                this.WindowState = FormWindowState.Normal;

            this.BringToFront();
            this.Activate();
        }

        private void OnFormClosing(object sender, FormClosingEventArgs e)
        {
            if (insertWatcher != null)
            {
                insertWatcher.Stop();
                insertWatcher.Dispose();
                insertWatcher = null;
            }
            if (removeWatcher != null)
            {
                removeWatcher.Stop();
                removeWatcher.Dispose();
                removeWatcher = null;
            }
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.Alt && e.KeyCode == Keys.L)
            {
                // Ensure password masking is active again
                // Enable input + mask again
                pswTextBox.Enabled = true;      // Ensure textbox is focusable
                pswTextBox.PasswordChar = '•';  // Use a safe character
                pswTextBox.Clear();             // Clear any previous input or message
                pswTextBox.Focus();

                lockButton.Enabled = false;
                unLockButton.Enabled = false;
                readButton.Enabled = true;

                if (this.blnLock == true)
                {
                    string driveLetter = driveComboBox.SelectedItem as string;
                    if (string.IsNullOrEmpty(driveLetter))
                    {
                        MessageBox.Show("Please select a USB device !", "Warning !", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                        // // Prevent any control from being focused
                        this.setNonActiveControl();
                        return;
                    }

                    // Ensure driveLetter has a trailing backslash (e.g., "E:\")
                    if (!driveLetter.EndsWith("\\"))
                        driveLetter += "\\";

                    DriveInfo driveToCheck = new DriveInfo(driveLetter);

                    if (!driveToCheck.IsReady)
                    {
                        MessageBox.Show("The selected drive is not ready !", "Error !", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                        // // Prevent any control from being focused
                        this.setNonActiveControl();

                        return;
                    }

                    this.pswTextBox.Enabled = true;
                    this.pswTextBox.Focus();
                }
                else
                {
                    pswTextBox.Clear();
                    pswTextBox.PasswordChar = '\0'; // Unmask to show success message
                    pswTextBox.Text = "Welcome to " + this.Text;
                    pswTextBox.Enabled = false;
                    pswTextBox.Focus();
                }

                this.blnLock = !this.blnLock;
            }

            if (e.KeyCode == Keys.F5)
            {
                // // Prevent any control from being focused
                this.setNonActiveControl();
            }
        }

        private void pswTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                //e.SuppressKeyPress = true; // Prevent beep sound

                if (driveComboBox.SelectedItem == null)
                {
                    ShowErrorAndReset("Please select a USB device !");
                    return;
                }

                // Get selected drive letter
                string selectedItem = driveComboBox.SelectedItem.ToString();
                string driveLetter = selectedItem.Split(' ')[0];
                if (!driveLetter.EndsWith("\\")) driveLetter += "\\";

                DriveInfo driveToCheck;
                try
                {
                    driveToCheck = new DriveInfo(driveLetter);
                }
                catch
                {
                    ShowErrorAndReset("Invalid drive selected !");
                    return;
                }

                if (!driveToCheck.IsReady)
                {
                    ShowErrorAndReset("The selected drive is not ready !");
                    return;
                }

                if (driveToCheck.DriveType != DriveType.Removable)
                {
                    ShowErrorAndReset("The selected drive is not a removable USB device !");
                    return;
                }

                string password = pswTextBox.Text.Trim();
                if (string.IsNullOrEmpty(password) || password != defaultAdminPass)
                {
                    lockButton.Enabled = false;
                    unLockButton.Enabled = false;
                    readButton.Enabled = true;

                    ShowErrorAndReset("Password is incorrect !");
                    return;
                }

                // ✅ Success
                lockButton.Enabled = true;
                unLockButton.Enabled = true;
                readButton.Enabled = true;

                pswTextBox.Clear();
                pswTextBox.PasswordChar = '\0'; // Unmask to show success message
                pswTextBox.Text = "You can Lock/UnLock USB device.";
                pswTextBox.Enabled = false;
                pswTextBox.Focus();
            }
        }

        private void RefreshDriveComboBox()
        {
            // Ensure UI update happens on UI thread
            if (InvokeRequired)
            {
                BeginInvoke(new MethodInvoker(RefreshDriveComboBox));
                return;
            }

            driveComboBox.Items.Clear();

            // Populate drive letters
            try
            {
                foreach (var drive in DriveInfo.GetDrives())
                {
                    if (drive.IsReady && drive.DriveType == DriveType.Removable)
                    {
                        driveComboBox.Items.Add(drive.Name.TrimEnd('\\') + " " + drive.VolumeLabel.ToString());
                    }
                }

                if (driveComboBox.Items.Count > 0) driveComboBox.SelectedIndex = 0; // select first drive by default
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message, "Error !", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                // // Prevent any control from being focused
                this.setNonActiveControl();
            }
            finally
            {
                if (driveComboBox.Items.Count > 0)
                {
                    driveComboBox.Enabled = true;
                    driveComboBox.SelectedIndex = 0;
                }
                else
                {
                    driveComboBox.Enabled = false;
                }
            }
        }

        private void ShowErrorAndReset(string message)
        {
            MessageBox.Show(message, "Warning !", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

            pswTextBox.Enabled = true;
            pswTextBox.Clear();

            // In .NET 4.0, no need for BeginInvoke unless you're cross-threading
            this.BeginInvoke(new Action(() => pswTextBox.Focus()));
        }

        #endregion

        #region USB Watchers Events

        private void StartUSBWatchers()
        {
            try
            {
                // USB Insert
                WqlEventQuery insertQuery = new WqlEventQuery(
                    "SELECT * FROM __InstanceCreationEvent WITHIN 2 WHERE TargetInstance ISA 'Win32_USBHub'"
                );
                insertWatcher = new ManagementEventWatcher(insertQuery);
                insertWatcher.EventArrived += new EventArrivedEventHandler(DeviceInserted);
                insertWatcher.Start();

                // USB Remove
                WqlEventQuery removeQuery = new WqlEventQuery(
                    "SELECT * FROM __InstanceDeletionEvent WITHIN 2 WHERE TargetInstance ISA 'Win32_USBHub'"
                );
                removeWatcher = new ManagementEventWatcher(removeQuery);
                removeWatcher.EventArrived += new EventArrivedEventHandler(DeviceRemoved);
                removeWatcher.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show("WMI Error: " + ex.Message, "Error !", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void DeviceInserted(object sender, EventArrivedEventArgs e)
        {
            this.Invoke((MethodInvoker)delegate
            {
                //MessageBox.Show("USB device inserted !", "USB Event", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RefreshDriveComboBox();
            });
        }

        private void DeviceRemoved(object sender, EventArrivedEventArgs e)
        {
            this.Invoke((MethodInvoker)delegate
            {
                //MessageBox.Show("USB device removed!", "USB Event", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RefreshDriveComboBox();
            });
        }

        #endregion

        #region USB Lock/UnLock

        private void ReadSector(object sender, EventArgs e, int sectorNumber = DEFAULT_SECTOR_NUMBER)
        {
            try
            {
                if (driveComboBox.SelectedItem == null)
                {
                    MessageBox.Show("Please select a USB device!", "Warning!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    setNonActiveControl();
                    return;
                }

                // Extract drive letter (e.g., "E:")
                string selectedItem = driveComboBox.SelectedItem.ToString();
                string driveLetter = selectedItem.Split(' ')[0]; // Assumes format like "E: USBNAME"

                if (string.IsNullOrEmpty(driveLetter))
                {
                    MessageBox.Show("Invalid drive selection!", "Warning!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    setNonActiveControl();
                    return;
                }

                // Get physical drive path
                string physicalPath = USBPhysicalDrive.GetPhysicalDrivePath(driveLetter);
                if (physicalPath == null)
                {
                    MessageBox.Show("USB device not found!", "Warning!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    setNonActiveControl();
                    return;
                }

                // Read sector (assuming USBRawReader.ReadSector returns a string or byte[] you convert)
                string sectorData = USBRawReader.ReadSector(physicalPath, sectorNumber);
                MessageBox.Show(sectorData, "USB data", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message, "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                setNonActiveControl();
            }
        }

        private void LockUSB(object sender, EventArgs e, int sectorNumber = DEFAULT_SECTOR_NUMBER)
        {
            try
            {
                // 1. Prepare sector data (locking signature)
                byte[] sectorData = new byte[512];

                byte[] inputData = new byte[] {
                    0xA2, 0xA0, 0xD0, 0xEB, 0xE5, 0xB9, 0x33, 0x44, 0x87, 0xC0, 0x68, 0xB6, 0xB7, 0x26, 0x99, 0xC7,
                    0x36, 0xF7, 0x38, 0x89, 0xB7, 0xA0, 0xE2, 0x11, 0xB0, 0x5A, 0x86, 0xF1, 0x22, 0xE5, 0x01, 0x2F,
                    0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x1F, 0xEE, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
                    0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
                    0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
                    0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
                };

                Array.Copy(inputData, sectorData, Math.Min(inputData.Length, sectorData.Length));

                // 2. Get drive letter and physical path
                string selectedItem = driveComboBox.SelectedItem.ToString();
                string driveLetter = selectedItem.Split(' ')[0]; // e.g., "E:"
                string physicalDrivePath = USBPhysicalDrive.GetPhysicalDrivePath(driveLetter);

                // 3. Get PNPDeviceID (used for reset)
                string pnpDeviceId = USBPhysicalDrive.GetPnpDeviceIdFromDriveLetter(driveLetter);

                if (!string.IsNullOrEmpty(physicalDrivePath))
                {
                    // 4. Write locking sector
                    bool writeSuccess = USBRawWriter.WriteSector(physicalDrivePath, sectorNumber, sectorData);
                    if (!writeSuccess)
                    {
                        MessageBox.Show("Failed to write locking sector.", "Write Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    // 5. Wait before ejecting
                    System.Threading.Thread.Sleep(1000);

                    // 6. Eject the drive via IOCTL
                    string physicalPath = USBPhysicalDrive.GetPhysicalDriveForLetter(driveLetter);
                    bool ejectSuccess = false;

                    if (!string.IsNullOrEmpty(physicalPath))
                    {
                        ejectSuccess = USBPhysicalDrive.EjectPhysicalDrive(physicalPath);
                    }

                    // 7. Show result
                    MessageBox.Show(
                        ejectSuccess ? "Lock USB device successfully." : "Eject failed or device is in use.",
                        ejectSuccess ? "Information" : "Warning",
                        MessageBoxButtons.OK,
                        ejectSuccess ? MessageBoxIcon.Information : MessageBoxIcon.Warning
                    );

                    //// 8. Optionally reset USB device (like in UnlockUSB)
                    //if (!string.IsNullOrEmpty(pnpDeviceId))
                    //{
                    //    bool resetSuccess = USBHelper2.FullyResetUsbDevice(pnpDeviceId);
                    //    if (!resetSuccess)
                    //    {
                    //        MessageBox.Show("Failed to reset USB device after locking.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    //    }
                    //}

                    //// Call this method when you want to force-mount all volumes without a drive letter
                    //USBHelper2.ForceMountVolumes();
                    //USBHelper2.AssignSpecificDriveLetter("U");

                    this.setNonActiveControl();
                }
                else
                {
                    MessageBox.Show("USB device not found!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    this.setNonActiveControl();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.setNonActiveControl();
            }
            finally
            {
                this.setNonActiveControl();
            }
        }

        private void UnLockUSB(object sender, EventArgs e, int sectorNumber = DEFAULT_SECTOR_NUMBER)
        {
            try
            {
                // 1. Prepare zero-filled sector
                byte[] zeroSector = new byte[512];

                // 2. Get drive letter and physical path
                string selectedItem = driveComboBox.SelectedItem.ToString();
                string driveLetter = selectedItem.Split(' ')[0]; // e.g., "E:"
                string physicalDrivePath = USBPhysicalDrive.GetPhysicalDrivePath(driveLetter);

                // 3. Get PNPDeviceID (used for reset)
                string pnpDeviceId = USBPhysicalDrive.GetPnpDeviceIdFromDriveLetter(driveLetter);

                if (!string.IsNullOrEmpty(physicalDrivePath))
                {
                    // 4. Write sector
                    bool writeSuccess = USBRawWriter.WriteSector(physicalDrivePath, sectorNumber, zeroSector);
                    if (!writeSuccess)
                    {
                        MessageBox.Show("Failed to write to sector.", "Write Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    // 5. Wait before continuing
                    System.Threading.Thread.Sleep(1000);

                    // 6. Eject the drive via IOCTL (PhysicalDriveEjector)
                    string physicalPath = USBPhysicalDrive.GetPhysicalDriveForLetter(driveLetter);
                    bool ejectSuccess = false;

                    if (!string.IsNullOrEmpty(physicalPath))
                    {
                        ejectSuccess = USBPhysicalDrive.EjectPhysicalDrive(physicalPath);
                    }

                    // 7. Show eject result
                    MessageBox.Show(
                        ejectSuccess ? "Unlock USB device successfully." : "Eject failed or device is in use.",
                        ejectSuccess ? "Information" : "Warning",
                        MessageBoxButtons.OK,
                        ejectSuccess ? MessageBoxIcon.Information : MessageBoxIcon.Warning
                    );

                    //// 8. Fully reset the USB device (eject, disable, re-enable)
                    //if (!string.IsNullOrEmpty(pnpDeviceId))
                    //{
                    //    bool resetResult = USBHelper2.FullyResetUsbDevice(pnpDeviceId);
                    //    MessageBox.Show(
                    //        resetResult ? "USB was re-enabled successfully." : "Failed to reset USB device.",
                    //        "Reset Result",
                    //        MessageBoxButtons.OK,
                    //        resetResult ? MessageBoxIcon.Information : MessageBoxIcon.Error
                    //    );
                    //}
                    //else
                    //{
                    //    MessageBox.Show("Unable to find PNPDeviceID to re-enable device.", "Device Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    //}

                    //// Call this method when you want to force-mount all volumes without a drive letter
                    //USBHelper2.ForceMountVolumes();
                    //USBHelper2.AssignSpecificDriveLetter("U");

                }
                else
                {
                    MessageBox.Show("USB device not found!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                // Clear focus or reset UI
                this.setNonActiveControl();
            }
        }

        #endregion

        #region NativeMethods, One Instant and Bring to top

        private static readonly int WM_SHOWME = NativeMethods.RegisterWindowMessage("USBWPRTC_SHOW_ME");
        protected override void WndProc(ref Message m)
        {
            if (m.Msg == WM_SHOWME)
            {
                if (this.WindowState == FormWindowState.Minimized)
                    this.WindowState = FormWindowState.Normal;

                this.Show();
                this.BringToFront();
                this.Activate();
                this.TopMost = true;
                this.TopMost = false;
            }

            base.WndProc(ref m);
        }
        internal static class NativeMethods
        {
            [DllImport("user32.dll")]
            public static extern int RegisterWindowMessage(string lpString);

            [DllImport("user32.dll", SetLastError = true)]
            public static extern bool PostMessage(IntPtr hWnd, int Msg, IntPtr wParam, IntPtr lParam);
        }

        #endregion
    }
}
